"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const fs_1 = require("fs");
const command_1 = require("@oclif/command");
class BaseCommand extends command_1.Command {
    printBanner(awsConfig, flags) {
        const awsRegion = awsConfig.region || flags.region;
        const awsProfile = awsConfig.credentials.profile || flags.profile;
        this.log(`
AWS Cost Saver
--------------
  Action: ${chalk_1.default.green(this.constructor.name)}
  AWS region: ${chalk_1.default.green(awsRegion)}
  AWS profile: ${chalk_1.default.green(awsProfile)}
  State file: ${flags['no-state-file']
            ? chalk_1.default.yellow('none')
            : chalk_1.default.green(flags['state-file'])}
  Dry run: ${flags['dry-run'] ? chalk_1.default.green('yes') : chalk_1.default.yellow('no')}
`);
    }
    async validateStateFilePath(flags) {
        if (!flags['no-state-file'] && fs_1.existsSync(flags['state-file'])) {
            this.log(chalk_1.default.yellow(`\n→ State file already exists: ${chalk_1.default.yellowBright(flags['state-file'])}\n`));
            const answers = await inquirer_1.default.prompt([
                {
                    name: 'stateFileOverwrite',
                    type: 'confirm',
                    default: false,
                    message: `Are you sure you want to ${chalk_1.default.bgRed(chalk_1.default.white('OVERWRITE'))} existing state-file?\n  You will not be able to restore to previously conserved resources!`,
                },
            ]);
            if (!answers.stateFileOverwrite) {
                this.log(chalk_1.default.redBright('\nIgnoring, to avoid overwriting state file! Use -n|--no-state-file flag to skip writing the state file.\n'));
                throw new Error('AbortedByUser');
            }
        }
    }
}
exports.default = BaseCommand;
//# sourceMappingURL=base-command.js.map