"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecreaseDynamoDBProvisionedRcuWcuTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
class DecreaseDynamoDBProvisionedRcuWcuTrick {
    constructor() {
        this.ddbClient = new aws_sdk_1.default.DynamoDB();
    }
    getMachineName() {
        return DecreaseDynamoDBProvisionedRcuWcuTrick.machineName;
    }
    getConserveTitle() {
        return 'Decrease DynamoDB Provisioned RCU and WCU';
    }
    getRestoreTitle() {
        return 'Restore DynamoDB Provisioned RCU and WCU';
    }
    async getCurrentState(task, currentState, options) {
        const tableNames = await this.listTableNames(task);
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (!tableNames || tableNames.length === 0) {
            task.skip('No DynamoDB tables found');
            return subListr;
        }
        subListr.add(tableNames.map((tableName) => {
            const tableState = {
                name: tableName,
            };
            currentState.push(tableState);
            return {
                title: tableName,
                task: async (ctx, task) => this.getTableState(task, tableState),
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const table of currentState) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(table.name)}`,
                    task: (ctx, task) => this.conserveTableProvisionedRcuWcu(task, table, options),
                });
            }
        }
        else {
            task.skip(`No DynamoDB tables found`);
        }
        return subListr;
    }
    async restore(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const table of currentState) {
                subListr.add({
                    title: `${chalk_1.default.blueBright(table.name)}`,
                    task: (ctx, task) => this.restoreTableProvisionedRcuWcu(task, table, options),
                });
            }
        }
        else {
            task.skip(`No DynamoDB tables was conserved`);
        }
        return subListr;
    }
    async listTableNames(task) {
        const tableNames = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        tableNames.push(...((await this.ddbClient.listTables({ Limit: 100 }).promise())
            .TableNames || []));
        return tableNames;
    }
    async getTableState(task, tableState) {
        var _a;
        task.output = 'Fetching table information...';
        const provisionedThroughput = (_a = (await this.ddbClient
            .describeTable({ TableName: tableState.name })
            .promise()).Table) === null || _a === void 0 ? void 0 : _a.ProvisionedThroughput;
        if (!provisionedThroughput) {
            tableState.provisionedThroughput = false;
            return;
        }
        tableState.provisionedThroughput = true;
        tableState.rcu = provisionedThroughput.ReadCapacityUnits || 1;
        tableState.wcu = provisionedThroughput.WriteCapacityUnits || 1;
    }
    async conserveTableProvisionedRcuWcu(task, tableState, options) {
        if (!tableState.provisionedThroughput) {
            task.skip(`Provisioned throughput is not configured`);
            return;
        }
        if (tableState.wcu < 2 && tableState.rcu < 2) {
            task.skip(`Provisioned RCU/WCU is already at minimum of 1`);
            return;
        }
        if (options.dryRun) {
            task.skip('Skipped, would configure RCU = 1 WCU = 1');
            return;
        }
        task.output = `Configuring RCU = 1 WCU = 1 ...`;
        await this.ddbClient
            .updateTable({
            TableName: tableState.name,
            ProvisionedThroughput: {
                ReadCapacityUnits: 1,
                WriteCapacityUnits: 1,
            },
        })
            .promise();
        task.output = `Configured RCU = 1 WCU = 1`;
    }
    async restoreTableProvisionedRcuWcu(task, tableState, options) {
        if (!tableState.provisionedThroughput) {
            task.skip(`Provisioned throughput was not configured`);
            return;
        }
        if (options.dryRun) {
            task.skip(`Skipped, would configure RCU = ${tableState.rcu} WCU = ${tableState.wcu}`);
            return;
        }
        if (!tableState.rcu || !tableState.wcu) {
            task.skip(`Skipped, RCU = ${tableState.rcu} WCU = ${tableState.wcu} are not configured correctly`);
            return;
        }
        task.output = `Configuring ${tableState.rcu} WCU = ${tableState.wcu} ...`;
        await this.ddbClient
            .updateTable({
            TableName: tableState.name,
            ProvisionedThroughput: {
                ReadCapacityUnits: tableState.rcu,
                WriteCapacityUnits: tableState.wcu,
            },
        })
            .promise();
        task.output = `Configured RCU = ${tableState.rcu} WCU = ${tableState.wcu}`;
    }
}
exports.DecreaseDynamoDBProvisionedRcuWcuTrick = DecreaseDynamoDBProvisionedRcuWcuTrick;
DecreaseDynamoDBProvisionedRcuWcuTrick.machineName = 'decrease-dynamodb-provisioned-rcu-wcu';
//# sourceMappingURL=decrease-dynamodb-provisioned-rcu-wcu.trick.js.map