"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScaledownAutoScalingGroupsTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class ScaledownAutoScalingGroupsTrick {
    constructor() {
        this.asClient = new aws_sdk_1.default.AutoScaling();
    }
    getMachineName() {
        return ScaledownAutoScalingGroupsTrick.machineName;
    }
    getConserveTitle() {
        return 'Scale-down Auto Scaling Groups';
    }
    getRestoreTitle() {
        return 'Restore Auto Scaling Groups';
    }
    async getCurrentState(task, currentState, options) {
        const scalingGroups = await this.listAutoScalingGroups(task);
        if (!scalingGroups || scalingGroups.length === 0) {
            task.skip('No ASG found');
            return;
        }
        currentState.push(...scalingGroups.map((asg) => {
            return {
                name: asg.AutoScalingGroupName,
                desired: asg.DesiredCapacity,
                min: asg.MinSize,
                max: asg.MaxSize,
            };
        }));
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const asgState of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${asgState.name}`),
                    task: (ctx, task) => this.conserveAutoScalingGroup(task, asgState, options),
                });
            }
        }
        else {
            task.skip(`No auto scaling groups found`);
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (originalState && originalState.length > 0) {
            for (const asgState of originalState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${asgState.name}`),
                    task: (ctx, task) => this.restoreAutoScalingGroup(task, asgState, options),
                });
            }
        }
        else {
            task.skip(`No auto scaling groups were conserved`);
        }
        return subListr;
    }
    async listAutoScalingGroups(task) {
        const groups = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        groups.push(...(await this.asClient
            .describeAutoScalingGroups({ MaxRecords: 100 })
            .promise()).AutoScalingGroups);
        return groups;
    }
    async conserveAutoScalingGroup(task, asgState, options) {
        if (options.dryRun) {
            task.skip('Skipped, would scale down the ASG desired = 0 min = 0 max = 0');
            return;
        }
        task.output = 'Scaling down ASG...';
        await this.asClient
            .updateAutoScalingGroup({
            AutoScalingGroupName: asgState.name,
            DesiredCapacity: 0,
            MinSize: 0,
            MaxSize: 0,
        })
            .promise();
        task.output = 'Scaled down';
    }
    async restoreAutoScalingGroup(task, asgState, options) {
        if (options.dryRun) {
            task.skip(`Skipped, would restore the ASG to desired = ${asgState.desired} min = ${asgState.min} max = ${asgState.max}`);
            return;
        }
        task.output = 'Restoring ASG...';
        await this.asClient
            .updateAutoScalingGroup({
            AutoScalingGroupName: asgState.name,
            DesiredCapacity: asgState.desired,
            MinSize: asgState.min,
            MaxSize: asgState.max,
        })
            .promise();
        task.output = `Restored`;
    }
}
exports.ScaledownAutoScalingGroupsTrick = ScaledownAutoScalingGroupsTrick;
ScaledownAutoScalingGroupsTrick.machineName = 'scaledown-auto-scaling-groups';
//# sourceMappingURL=scaledown-auto-scaling-groups.trick.js.map