"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SuspendAutoScalingGroupsTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const listr_1 = tslib_1.__importDefault(require("listr"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class SuspendAutoScalingGroupsTrick {
    constructor() {
        this.asClient = new aws_sdk_1.default.AutoScaling();
    }
    getMachineName() {
        return SuspendAutoScalingGroupsTrick.machineName;
    }
    getConserveTitle() {
        return 'Suspend Auto Scaling Groups';
    }
    getRestoreTitle() {
        return 'Resume Auto Scaling Groups';
    }
    async getCurrentState(task, currentState, options) {
        const scalingGroups = await this.listAutoScalingGroups(task);
        if (!scalingGroups || scalingGroups.length === 0) {
            task.skip('No ASG found');
            return;
        }
        currentState.push(...scalingGroups.map((asg) => {
            return {
                name: asg.AutoScalingGroupName,
            };
        }));
    }
    async conserve(task, currentState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (currentState && currentState.length > 0) {
            for (const asgState of currentState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${asgState.name}`),
                    task: (ctx, task) => this.conserveProcesses(task, asgState, options),
                });
            }
        }
        else {
            task.skip(`No auto scaling groups found`);
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = new listr_1.default({
            concurrent: 10,
            exitOnError: false,
            collapse: false,
        });
        if (originalState && originalState.length > 0) {
            for (const asgState of originalState) {
                subListr.add({
                    title: chalk_1.default.blueBright(`${asgState.name}`),
                    task: (ctx, task) => this.restoreProcesses(task, asgState, options),
                });
            }
        }
        else {
            task.skip(`No auto scaling groups were conserved`);
        }
        return subListr;
    }
    async listAutoScalingGroups(task) {
        const groups = [];
        // TODO Add logic to go through all pages
        task.output = 'Fetching page 1...';
        groups.push(...(await this.asClient
            .describeAutoScalingGroups({ MaxRecords: 100 })
            .promise()).AutoScalingGroups);
        return groups;
    }
    async conserveProcesses(task, asgState, options) {
        if (options.dryRun) {
            task.skip('Skipped, would suspend ASG processes');
            return;
        }
        task.output = 'Suspending ASG processes...';
        await this.asClient
            .suspendProcesses({
            AutoScalingGroupName: asgState.name,
        })
            .promise();
        task.output = 'Suspended';
    }
    async restoreProcesses(task, asgState, options) {
        if (options.dryRun) {
            task.skip(`Skipped, would resume ASG processes`);
            return;
        }
        task.output = 'Resuming ASG...';
        await this.asClient
            .resumeProcesses({
            AutoScalingGroupName: asgState.name,
        })
            .promise();
        task.output = `Restored`;
    }
}
exports.SuspendAutoScalingGroupsTrick = SuspendAutoScalingGroupsTrick;
SuspendAutoScalingGroupsTrick.machineName = 'suspend-auto-scaling-groups';
//# sourceMappingURL=suspend-auto-scaling-groups.trick.js.map